import pandas as pd


def load_and_preprocess_data(df: pd.DataFrame = None,
                             ratings_path: str = './src/rec_system/data/ratings.csv') -> pd.DataFrame:
    """Converts results to be compatible with recommendation model.
                Parameters:
                    df (pd.DataFrame): dataframe with results from the poll
                    ratings_path (str): if dataframe was not provided or is None this is a path with csv with all ratings from the system
                Returns:
                    (pd.DataFrame) Dataframe with results modified results form poll.
    """
    if df is None:
        df = pd.read_csv(ratings_path)
    if df.empty:
        print("no data found")
        return None
    ratings_matrix = df.pivot_table(values='ocena', index='Adres e-mail', columns='pytanie', fill_value=0)
    X = ratings_matrix.T
    return X



import pandas as pd

from src.rec_system.engine.recommender import Recommender
from src.rec_system.utils import load_and_preprocess_data
import pytest


class TestRecommender:
    """
    Tests related to Recommender
    """

    @pytest.mark.parametrize("test_input,expected", [
        (('tast@gmail.com', 0.7, pd.read_csv('./tests/integration/Data/user_profiles.csv'),
          pd.read_csv('./src/rec_system/data/recipes.csv')), (list, int))])
    def test_product_recommendation_type(self, test_input, expected):
        data = load_and_preprocess_data()
        recommender = Recommender(data)
        recommender.create_and_fit()
        recommendations = recommender.recommend_products(test_input[0], test_input[1], test_input[2],
                                                         test_input[3])
        assert expected[0] == type(recommendations)
        for recommendation in recommendations:
            assert expected[1] == type(recommendation)

    @pytest.mark.parametrize("test_input,expected", [
        (('Corned Beef and Cabbage', 0.7,
          pd.read_csv('./src/rec_system/data/recipes.csv')), (list, int))])
    def test_similar_product_recommendation_type(self, test_input, expected):
        data = load_and_preprocess_data()
        recommender = Recommender(data)
        recommender.create_and_fit()
        recommendations = recommender.similar_dishes(test_input[0], test_input[1], test_input[2], )
        assert expected[0] == type(recommendations)
        for recommendation in recommendations:
            assert expected[1] == type(recommendation)

    @pytest.mark.parametrize("test_input,expected", [
        (('foo', 0.7,
          pd.read_csv('./src/rec_system/data/recipes.csv')), ValueError)])
    def test_similar_product_recommendation_wrong_value(self, test_input, expected):
        data = load_and_preprocess_data()
        recommender = Recommender(data)
        recommender.create_and_fit()

    @pytest.mark.parametrize("test_input,expected", [
        (('foo_email', 0.7, pd.read_csv('./tests/integration/Data/user_profiles.csv'),
          pd.read_csv('./src/rec_system/data/recipes.csv')), KeyError)])
    def test_product_recommendation_wrong_value(self, test_input, expected):
        data = load_and_preprocess_data()
        recommender = Recommender(data)
        recommender.create_and_fit()
        with pytest.raises(expected) as exc_info:
            recommender.recommend_products(test_input[0], test_input[1], test_input[2],
                                           test_input[3])
